#include "event.h"
#include "utils.h"
#include <stdlib.h>
#include <stdio.h>

static queue q;

void event_create(void)
{
  init_queue(&q);
}

void init_event(event *e)
{
  e->data = NULL;
}

void wait_event(event *e)
{
  event *d = wait_dequeue(&q);
  if (e->data) free(e->data);
  *e = *d;
  free(d);
}

void send_event(enum event_type type, void *data)
{
  event *d = malloc(sizeof(event)); if (d == NULL) abort();
  d->type = type;
  d->data = data;
  enqueue(&q, d);
}

void set_event(event *e, enum event_type t, void *d)
{
  e->type = t;
  e->data = d;
}

char *event2s(event *e)
{
  static char s[512];
  switch (e->type) {
  case SIP_READY: sprintf(s, "SIP_READY"); break;
  case CALLING: sprintf(s, "CALLING %s", (char*)e->data); break;
  case CALLED: sprintf(s, "CALLED by %s", (char*)e->data); break;
  case ENDED: sprintf(s, "ENDED"); break;
  case STOP: sprintf(s, "STOP"); break;
  case ACCEPTED: sprintf(s, "ACCEPTED"); break;
  case UNACCEPTED: sprintf(s, "UNACCEPTED"); break;
  case ACCEPT: sprintf(s, "ACCEPT"); break;
  case REJECT: sprintf(s, "REJECT"); break;
  case QUIT: sprintf(s, "QUIT"); break;
  }
  return s;
}
