#include "event.h"
#include "phone.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

void usage(void)
{
  printf("phone <options>\n");
  printf("mandatory options:\n");
  printf("    -u <user>\n");
  printf("    -p <password>\n");
  printf("    -i <public IP>\n");
  printf("optional options:\n");
  printf("    -s <SIP port (default 8080)>\n");
  printf("    -r <RTP port (default 8081)>\n");
  printf("    -ai <input audio device, eg. hw:2 (default default)>\n");
  printf("    -am <input mode # (default 0)>\n");
  printf("        0: S16_LE, 16HKz, mono\n");
  printf("        1: S16_LE, 48KHz, stereo, use left channel\n");
  exit(0);
}

int main(int n, char **v)
{
  int i;
  char *user = NULL;
  char *pass = NULL;
  char *public_ip = NULL;
  int sip_port = 8080;
  int rtp_port = 8081;
  char *audio_device = "default";
  int audio_mode = 0;
  event e;

  for (i = 1; i < n; i++) {
    if (i > n - 2) usage();
    if(!strcmp(v[i],"-u")) {i++;user        =v[i];continue;}
    if(!strcmp(v[i],"-p")) {i++;pass        =v[i];continue;}
    if(!strcmp(v[i],"-i")) {i++;public_ip   =v[i];continue;}
    if(!strcmp(v[i],"-s")) {i++;sip_port    =atoi(v[i]);continue;}
    if(!strcmp(v[i],"-r")) {i++;rtp_port    =atoi(v[i]);continue;}
    if(!strcmp(v[i],"-ai")){i++;audio_device=v[i];continue;}
    if(!strcmp(v[i],"-am")){i++;audio_mode  =atoi(v[i]);continue;}
    usage();
  }

  if (user == NULL || pass == NULL || public_ip == NULL) usage();

  event_create();
  sip_create(user, pass, public_ip, sip_port, rtp_port);
  rtp_create();
  audio_create(audio_device, audio_mode);
  keyboard_create();
  init_event(&e);

  printf("wait for sip\n");
  while (1) {
    wait_event(&e);
    if (e.type == SIP_READY) goto wait;
    if (e.type == QUIT) return 0;
    printf("unwanted event %s\n", event2s(&e));
  }

wait:
  printf("waiting, [c]all <X>, [q]uit\n");
  while (1) {
    wait_event(&e);
    if (e.type == CALLING) goto calling;
    if (e.type == CALLED) goto called;
    if (e.type == QUIT) goto quit;
    printf("unwanted event %s\n", event2s(&e));
  }

in_call:
  printf("call running, [s]top\n");
  rtp_start(rtp_port, sip_rtp_ip(), sip_rtp_port());
  audio_start();
  while (1) {
    wait_event(&e);
    if (e.type == ENDED) break;
    if (e.type == STOP) break;
    if (e.type == QUIT) { sip_stop(); goto quit; }
    printf("unwanted event %s\n", event2s(&e));
  }
  rtp_stop();
  audio_stop();
  sip_stop();
  goto wait;

calling:
  printf("calling %s\n", (char*)e.data);
  sip_call(e.data);
  while (1) {
    wait_event(&e);
    if (e.type == ACCEPTED) goto in_call;
    if (e.type == UNACCEPTED) goto wait;
    if (e.type == QUIT) goto quit;
    printf("unwanted event %s\n", event2s(&e));
  }

called:
  printf("%s is calling you, [a]ccept, [r]eject\n", (char*)e.data);
  while (1) {
    wait_event(&e);
    if (e.type == ACCEPT) { sip_accept(); goto in_call; }
    if (e.type == REJECT) { sip_reject(); goto wait; }
    if (e.type == QUIT) goto quit;
    printf("unwanted event %s\n", event2s(&e));
  }

quit:
  printf("closing sip\n");
  sip_close();
  while (1) {
    wait_event(&e);
    if (e.type == SIP_READY) break;
    printf("unwanted event %s\n", event2s(&e));
  }

  return 0;
}
