#include "phone.h"
#include "utils.h"
#include <stdlib.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdint.h>

static lockwait lock;
static volatile int run;
static int local_port;
static char *rtp_ip;
static int remote_port;

static struct sockaddr_in other;
static int s;

uint32_t bits(unsigned char *c, int first, int last)
{
  uint32_t v = (c[0] << 24) | (c[1] << 16) | (c[2] << 8) | c[3];
  int nbits = last-first+1;
  uint32_t mask;
  if (nbits == 32) return v;
  v >>= 31-last;
  mask = (1<<nbits) - 1;
  return v & mask;
}

static void *rtp_thread(void *_)
{
  unsigned char buf[4096];

start:
  lw_lock(&lock);
  while (!run) lw_wait(&lock);
  lw_unlock(&lock);

printf("thread run!\n");

  while (run) {
    ssize_t len = recvfrom(s, buf, 4096, 0, NULL, NULL);
    int v, p, x, cc, m, pt, seqnum;
    uint32_t timestamp;
    uint32_t ssrc;
    if (len < 12) { printf("bad rtp packet\n"); continue; }

    v = bits(buf, 0, 1);
    p = bits(buf, 2, 2);
    x = bits(buf, 3, 3);
    cc = bits(buf, 4, 7);
    m = bits(buf, 8, 8);
    pt = bits(buf, 9, 15);
    seqnum = bits(buf, 16, 31);

    timestamp = bits(buf+4, 0, 31);

    ssrc = bits(buf+8, 0, 31);

    if (cc) { printf("rtp packet with cc = %d\n", cc); continue; }
    if (pt == 1) continue;

    audio_speex_play((char*)buf+12, len-12);
//    printf("v %d p %d x %d cc %d m %d pt %d seqnum %d tstamp %d ssrc %d\n",
//           v, p, x, cc, m, pt, seqnum, timestamp, ssrc);
  }

  goto start;

  return 0;
}

static void *rtp_recthread(void *_)
{
  unsigned char buf[4096];
  int seqnum;
  uint32_t timestamp;
  uint32_t ssrc;

start:
  lw_lock(&lock);
  while (!run) lw_wait(&lock);
  lw_unlock(&lock);

printf("recthread run!\n");

  audio_open_record();

  seqnum = 0;
  timestamp = 0;
  ssrc = 666;

  while (run) {
    ssize_t len;

    buf[0] = 0x80;
    buf[1] = 111;
    *(int16_t *)(buf+2) = htons(seqnum);
    *(uint32_t *)(buf+4) = htonl(timestamp);
    *(uint32_t *)(buf+8) = htonl(ssrc);

    len = audio_speex_record((char*)buf+12, 4096-12) + 12;
    if (len > 12)
    if (sendto(s, buf, len, 0, (struct sockaddr *)&other, sizeof(other))!=len)
      printf("error sending audio\n");

if (0) {
    int v = bits(buf, 0, 1);
    int p = bits(buf, 2, 2);
    int x = bits(buf, 3, 3);
    int cc = bits(buf, 4, 7);
    int m = bits(buf, 8, 8);
    int pt = bits(buf, 9, 15);
    int seqnum = bits(buf, 16, 31);

    int timestamp = bits(buf+4, 0, 31);

    int ssrc = bits(buf+8, 0, 31);

    printf("len %ld ME v %d p %d x %d cc %d m %d pt %d seqnum %d tstamp %d ssrc %d\n",
           len, v, p, x, cc, m, pt, seqnum, timestamp, ssrc);
}
    seqnum++;
    timestamp += 320;
  }

  audio_close_record();

  goto start;

  return 0;
}

void rtp_create(void)
{
  lw_init(&lock);
  new_thread(rtp_thread, NULL);
  new_thread(rtp_recthread, NULL);
}

void rtp_start(int _local_port, char *_rtp_ip, int _remote_port)
{
  struct sockaddr_in addr;

  local_port = _local_port;
  if (rtp_ip != NULL) free(rtp_ip);
  rtp_ip = _rtp_ip;
  remote_port = _remote_port;

  other.sin_family = AF_INET;
  other.sin_port = htons(remote_port);
  other.sin_addr.s_addr = inet_addr(rtp_ip);
  printf("info: if remote IP not x.x.x.x, call won't work: %s\n", rtp_ip);

  s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if (s == -1) { perror("socket"); abort(); }

  addr.sin_family = AF_INET;
  addr.sin_port = htons(local_port);
  addr.sin_addr.s_addr = INADDR_ANY; //inet_addr("0.0.0.0");
  if (bind(s, (struct sockaddr *)&addr, sizeof(addr)))
    { perror("bind"); abort(); }

  run = 1;
  lw_wakeup(&lock);
}

void rtp_stop(void)
{
  lw_lock(&lock);
  run = 0;
  close(s);
  s = -1;
  lw_unlock(&lock);
}
