#include "phone.h"
#include "event.h"
#include "utils.h"
#include <stdio.h>
#include <eXosip2/eXosip.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <string.h>

static struct eXosip_t *ctx;
/* not sure at all with those cid/did/rid/tid things */
static int cid;
static int did;
static int rid;
static int tid;

static char *user;
static char *pass;
static char *public_ip;
static int sip_port;
static int rtp_port;
static int rtp_remote_port;
static char *rtp_remote_ip;

static void *sip_thread(void *_)
{
  eXosip_event_t *evt;

//  TRACE_INITIALIZE (6, NULL);
  ctx = eXosip_malloc(); if (ctx == NULL) abort();
  if (eXosip_init(ctx)) abort();
  if (eXosip_listen_addr(ctx,IPPROTO_UDP,NULL,sip_port,AF_INET,0)) abort();
  eXosip_masquerade_contact (ctx, public_ip, sip_port);

  eXosip_lock (ctx);
  eXosip_add_authentication_info (ctx, user, user, pass, NULL, NULL);
  eXosip_unlock (ctx);

  eXosip_lock(ctx);
  osip_message_t *reg = NULL;
  {
    char s[512];
    sprintf(s, "sip:%s@sip.linphone.org", user);
    rid = eXosip_register_build_initial_register(ctx, s,
                   "sip:sip.linphone.org:5060", NULL, 1800, &reg);
    if (rid<0) abort();
    if (eXosip_register_send_register (ctx, rid, reg)) abort();
  }
  eXosip_unlock(ctx);

  while (1) {
    evt = eXosip_event_wait (ctx, 10, 0);
    eXosip_lock(ctx);
    eXosip_automatic_action(ctx);
    eXosip_unlock(ctx);
    if (evt == NULL) continue;
printf("EV %d ######################################## %d\n", evt->type,
       evt->response ? osip_message_get_status_code(evt->response) : -1);
    switch (evt->type) {
    default: printf("############ '%s'\n", evt->textinfo); break;
    case EXOSIP_CALL_INVITE: {
      sdp_message_t *m;
      char *name;
      cid = evt->cid;
      did = evt->did;
//      rid = evt->rid;
      tid = evt->tid;
      eXosip_lock(ctx);
      eXosip_call_send_answer(ctx, evt->tid, 180, NULL);
      eXosip_unlock(ctx);
      m = eXosip_get_remote_sdp(ctx, evt->did);
      name = strdup(m->o_username); if (name == NULL) abort();
      send_event(CALLED, name);
      break;
    }
    case EXOSIP_REGISTRATION_SUCCESS: send_event(SIP_READY, NULL); break;
    case EXOSIP_REGISTRATION_FAILURE:
      if (evt->response && osip_message_get_status_code(evt->response)==403){
        printf("registration failed\n");
        send_event(QUIT, NULL);
      }
      break;
    case EXOSIP_CALL_RINGING: printf("ringing (%s)\n", evt->textinfo); break;
    case EXOSIP_CALL_REQUESTFAILURE:
      if (evt->response && osip_message_get_status_code(evt->response) == 404) {
        printf("error 404, not found\n");
        send_event(UNACCEPTED, NULL);
      }
      break;
    case EXOSIP_CALL_ANSWERED: {
      char tt[2048];
      osip_message_t *ack;
      sdp_message_t *m;
      sdp_connection_t *t;
      sdp_media_t *med;
      cid = evt->cid;
      did = evt->did;
//      rid = evt->rid;
      tid = evt->tid;
      m = eXosip_get_remote_sdp(ctx, did);
      t = eXosip_get_audio_connection(m);
      med = eXosip_get_audio_media(m);
      rtp_remote_port = atoi(med->m_port);
      sprintf(tt, "v=0\r\n"
          "o=sxdx2 0 0 IN IP4 %s\r\n"
          "s=Talk\r\n"
          "c=IN IP4 %s\r\n"
          "t=0 0\r\n"
          "m=audio 8081 RTP/AVP 111\r\n"
          "a=rtpmap:111 speex/16000\r\n"
          "a=fmtp:111 vbr=on\r\n",
          public_ip, public_ip);
     if (eXosip_call_build_ack(ctx, evt->did, &ack)) abort();
     osip_message_set_body(ack, tt, strlen(tt));
     osip_message_set_content_type(ack, "application/sdp");
     eXosip_lock(ctx);
     if (eXosip_call_send_ack(ctx, evt->did, ack)) abort();
     eXosip_unlock(ctx);
//      if (rtp_remote_ip) free(rtp_remote_ip);
     rtp_remote_ip = strdup(t->c_addr); if (rtp_remote_ip == NULL) abort();
#if 0
printf("c_nettype %s c_addrtype %s c_addr %s c_addr_multicast_ttl %s c_addr_multicast_int %s\n",
      t->c_nettype, t->c_addrtype, t->c_addr, t->c_addr_multicast_ttl, t->c_addr_multicast_int);
printf("m_port %s\n",
       med->m_port);
osip_list_t l = med->c_connections;
printf("list %d elements\n", l.nb_elt);
__node_t *nd = l.node;
while (nd) {
  sdp_connection_t *t = nd->element;
printf("c_nettype %s c_addrtype %s c_addr %s c_addr_multicast_ttl %s c_addr_multicast_int %s\n",
      t->c_nettype, t->c_addrtype, t->c_addr, t->c_addr_multicast_ttl, t->c_addr_multicast_int);
  nd = nd->next;
}
#endif
      send_event(ACCEPTED, NULL);
      break;
    }
    case EXOSIP_CALL_CLOSED:
      printf("call ended by other\n");
      send_event(ENDED, NULL);
      break;
    }
    eXosip_event_free(evt);
  }

  return 0;
}

void sip_create(char *_user, char *_pass, char *_public_ip,
                int _sip_port, int _rtp_port)
{
  user = _user;
  pass = _pass;
  public_ip = _public_ip;
  sip_port = _sip_port;
  rtp_port = _rtp_port;
  new_thread(sip_thread, NULL);
}

char *sip_rtp_ip(void)
{
  return rtp_remote_ip;
}

int sip_rtp_port(void)
{
  return rtp_remote_port;
}

void sip_call(char *other)
{
  char s[512];
  char t[2048];
  osip_message_t *invite;

  sprintf(s, "<sip:%s@sip.linphone.org>", other);
  sprintf(s+256, "<sip:%s@sip.linphone.org>", user);
  if (eXosip_call_build_initial_invite (ctx, &invite, s, s+256,
                                        NULL, // optional route header
                                        "call")) abort();
  sprintf(t,
      "v=0\r\n"
      "o=%s 0 0 IN IP4 %s\r\n"
      "s=Talk\r\n"
      "c=IN IP4 %s\r\n"
      "t=0 0\r\n"
      "m=audio %d RTP/AVP 111\r\n"
      "a=rtpmap:111 speex/16000\r\n"
      "a=fmtp:111 vbr=on\r\n",
      user, public_ip, public_ip, rtp_port);

  osip_message_set_body(invite, t, strlen(t));
  osip_message_set_content_type(invite, "application/sdp");
  eXosip_lock (ctx);
  cid = eXosip_call_send_initial_invite (ctx, invite); if (cid < 0) abort();
  eXosip_unlock (ctx);
}

void sip_accept(void)
{
  int i;
  char t[2048];
  sdp_message_t *m;
  sdp_connection_t *mt;
  sdp_media_t *med;
  osip_message_t *answer = NULL;
  m = eXosip_get_remote_sdp(ctx, did);
  mt = eXosip_get_audio_connection(m);
  med = eXosip_get_audio_media(m);
  rtp_remote_port = atoi(med->m_port);
  rtp_remote_ip = strdup(mt->c_addr); if (rtp_remote_ip == NULL) abort();
  sprintf(t,
      "v=0\r\n"
      "o=%s 0 0 IN IP4 %s\r\n"
      "s=Talk\r\n"
      "c=IN IP4 %s\r\n"
      "t=0 0\r\n"
      "m=audio %d RTP/AVP 111\r\n"
      "a=rtpmap:111 speex/16000\r\n"
      "a=fmtp:111 vbr=on\r\n",
      user, public_ip, public_ip, rtp_port);
  eXosip_lock (ctx);
  i = eXosip_call_build_answer (ctx, tid, 200, &answer);
  if (i != 0) {
    eXosip_call_send_answer (ctx, tid, 400, NULL);
  } else {
    osip_message_set_body(answer, t, strlen(t));
    osip_message_set_content_type(answer, "application/sdp");
    eXosip_call_send_answer (ctx, tid, 200, answer);
  }
  eXosip_unlock (ctx);
}

void sip_reject(void)
{
  eXosip_lock (ctx);
  eXosip_call_send_answer (ctx, tid, 603, NULL);
  eXosip_unlock (ctx);
}

void sip_close(void)
{
  osip_message_t *reg = NULL;
  eXosip_lock (ctx);
  if (eXosip_register_build_register (ctx, rid, 0, &reg) < 0) abort();
  eXosip_register_send_register (ctx, rid, reg);
  eXosip_unlock (ctx);
}

void sip_stop(void)
{
  eXosip_lock (ctx);
  eXosip_call_terminate (ctx, cid, did);
  eXosip_unlock (ctx);
}
